<?php
class DB
{
	public static function CurrentConnection()
	{
		global $currentConnection;
		return $currentConnection ? $currentConnection : DB::DefaultConnection();
	}

	public static function CurrentConnectionId()
	{
		$conn = DB::CurrentConnection();
		return $conn->connId;
	}

	public static function DefaultConnection()
	{
		global $cman;
		return $cman->getDefault();
	}

	public static function ConnectionByTable( $table )
	{
		global $cman;
		return $cman->byTable($table);
	}

	public static function ConnectionByName( $name )
	{
		global $cman;
		return $cman->byName( $name );
	}

	public static function SetConnection( $connection )
	{
		global $currentConnection;
		if ( is_string( $connection ) )
		{
			$currentConnection = DB::ConnectionByName( $connection );
		}
		else if ( is_a($connection, 'Connection') ) {
		 	$currentConnection = $connection;
		}
	}

	public static function LastId()
	{
		return DB::CurrentConnection()->getInsertedId();
	}

	public static function Query( $sql )
	{
		return DB::CurrentConnection()->querySilent( $sql );
	}

	public static function Exec( $sql )
	{
		return DB::CurrentConnection()->execSilent( $sql ) != NULL;
	}

	public static function LastError()
	{
		return DB::CurrentConnection()->lastError();
	}

	public static function Select($table, $userConditions = array() )
	{
		$tableInfo = DB::_getTableInfo($table);

		if ( !$tableInfo )
			return false;

		$whereSql = DB::_getWhereSql($userConditions, $tableInfo["fields"]);

		$sql = "SELECT * FROM ".DB::CurrentConnection()->addTableWrappers( $tableInfo['fullName'] ) . $whereSql;
		$queryResult = DB::CurrentConnection()->querySilent( $sql );

		return $queryResult;
	}

	public static function Delete($table, $userConditions = array() )
	{
		$tableInfo = DB::_getTableInfo($table);

		if ( !$tableInfo )
			return false;

		$whereSql = DB::_getWhereSql($userConditions, $tableInfo["fields"]);

		if( $whereSql == "" )
			return false;

		$sql = "DELETE FROM ".DB::CurrentConnection()->addTableWrappers( $tableInfo['fullName'] ) . $whereSql;
		$ret = DB::CurrentConnection()->execSilent( $sql );

		return $ret;
	}

	public static function Insert($table, $data)
	{
		$result = false;
		$tableInfo = DB::_getTableInfo($table);

		if ( !$tableInfo )
			return false;

		$iFields = "";
		$iValues = "";
		$blobs = array();
		foreach($data as $fieldname => $value)
		{
			$field = getArrayElementNC($tableInfo["fields"], $fieldname);

			// user field not found in table
			if ( is_null($field) )
				continue;

			$iFields.= DB::CurrentConnection()->addFieldWrappers( $field["name"] ).",";
			$iValues.= DB::_prepareValue($value, $field["type"]) . ",";

			if( DB::CurrentConnection()->dbType == nDATABASE_Oracle || DB::CurrentConnection()->dbType == nDATABASE_DB2 || DB::CurrentConnection()->dbType == nDATABASE_Informix )
			{
				if( IsBinaryType( $field["type"] ) )
					$blobs[ $field["name"] ] = $value;

				if( DB::CurrentConnection()->dbType == nDATABASE_Informix && IsTextType( $field["type"] ) )
					$blobs[ $field["name"] ] = $value;
			}
		}

		if( $iFields != "" && $iValues != "" )
		{
			$iFields = substr($iFields, 0, -1);
			$iValues = substr($iValues, 0, -1);
			$sql = "INSERT INTO ".DB::CurrentConnection()->addTableWrappers( $tableInfo['fullName'] )." (".$iFields.") values (".$iValues.")";

			if ( count($blobs) > 0 )
				$result = DB::_execSilentWithBlobProcessing($blobs, $sql, $tableInfo['fields']);
			else
				$result = DB::CurrentConnection()->execSilent( $sql );
		}

		return $result;
	}

	public static function Update($table, $data, $userConditions)
	{
		$result = false;
		$tableInfo = DB::_getTableInfo($table);

		if ( !$tableInfo )
			return false;

		$whereSql = DB::_getWhereSql($userConditions, $tableInfo["fields"]);

		if( $whereSql == "" )
			return false;

		$updateValues = array();
		$blobs = array();
		foreach( $data as $fieldname => $value )
		{
			$field = getArrayElementNC($tableInfo["fields"], $fieldname);

			// user field not found in table
			if ( is_null($field) )
				continue;

			$prepareFieldName = DB::CurrentConnection()->addFieldWrappers( $field["name"] );
			$prepareValue = DB::_prepareValue($value, $field["type"]);
			$updateValues[] = $prepareFieldName."=".$prepareValue;

			if ( DB::CurrentConnection()->dbType == nDATABASE_Oracle || DB::CurrentConnection()->dbType == nDATABASE_DB2 || DB::CurrentConnection()->dbType == nDATABASE_Informix )
			{
				if ( IsBinaryType( $field["type"] ) )
					$blobs[ $field["name"] ] = $value;

				if ( DB::CurrentConnection()->dbType == nDATABASE_Informix && IsTextType( $field["type"] ) )
					$blobs[ $field["name"] ] = $value;
			}
		}

		if ( count($updateValues) > 0 )
		{
			$updateSQL = implode(",", $updateValues);
			$sql = "UPDATE ".DB::CurrentConnection()->addTableWrappers( $tableInfo['fullName'] )." SET ". $updateSQL . $whereSql;

			if ( count($blobs) > 0 )
				$result = DB::_execSilentWithBlobProcessing($blobs, $sql, $tableInfo['fields']);
			else
				$result = DB::CurrentConnection()->execSilent( $sql );
		}

		return $result;
	}

	protected static function _getWhereSql($userConditions, $founedfields)
	{
		if( !is_array( $userConditions ) )
		{
			$whereSql = trim( $userConditions );
			if( $whereSql != "")
				$whereSql = " WHERE " . $whereSql;
			return $whereSql;
		}

		$conditions = array();
		foreach($userConditions as $fieldname => $value)
		{
			$field = getArrayElementNC($founedfields, $fieldname);
			// user field not found in table
			if ( is_null($field) )
				continue;

			$wrappedField = DB::CurrentConnection()->addFieldWrappers( $field["name"] );
			if ( is_null($value) )
			{
				$conditions[] = $wrappedField . " IS NULL";
			}
			else
			{
				$conditions[] = $wrappedField . "=" . DB::_prepareValue($value, $field["type"]);
			}
		}

		$whereSql = "";
		if( count($conditions) > 0 )
		{
			$whereSql .= " WHERE " . implode(" AND ", $conditions);
		}

		return $whereSql;
	}

	/**
	 * @param Array blobs
	 * @param String dalSQL
	 * @param Array tableinfo
	 */
	protected static function _execSilentWithBlobProcessing($blobs, $dalSQL, $tableinfo)
	{
		$blobTypes = array();
		if( DB::CurrentConnection()->dbType == nDATABASE_Informix )
		{
			foreach( $blobs as $fname => $fvalue )
			{
				$blobTypes[ $fname ] = $tableinfo[ $fname ]["type"];
			}
		}

		DB::CurrentConnection()->execSilentWithBlobProcessing( $dalSQL, $blobs, $blobTypes );
	}

	protected static function _prepareValue($value, $type)
	{
		if ( is_null($value) )
			return "NULL";

		if( DB::CurrentConnection()->dbType == nDATABASE_Oracle || DB::CurrentConnection()->dbType == nDATABASE_DB2 || DB::CurrentConnection()->dbType == nDATABASE_Informix )
		{
			if( IsBinaryType($type) )
			{
				if( DB::CurrentConnection()->dbType == nDATABASE_Oracle )
					return "EMPTY_BLOB()";

				return "?";
			}

			if( DB::CurrentConnection()->dbType == nDATABASE_Informix  && IsTextType($type) )
				return "?";
		}

		if( IsNumberType($type) && !is_numeric($value) )
		{
			$value = trim($value);
			$value = str_replace(",", ".", $value);
			if ( !is_numeric($value) )
				return "NULL";
		}

		if( IsDateFieldType($type) || IsTimeType($type) )
		{
			if( !$value )
				return "NULL";

			// timestamp
			if ( is_int($value) )
			{
				if ( IsDateFieldType($type) )
				{
					$value = getYMDdate($value) . " " . getHISdate($value);
				}
				else if ( IsTimeType($type) )
				{
					$value = getHISdate($value);
				}
			}

			return DB::CurrentConnection()->addDateQuotes( $value );
		}

		if( NeedQuotes($type) )
			return DB::CurrentConnection()->prepareString( $value );

		return $value;
	}

	/**
	 * 	Find table info stored in the project file
	 *
	 */
	public static function _findDalTable( $table, $conn = null )
	{
		global $dalTables;
		if( !$conn )
			$conn = DB::CurrentConnection();
		$tableName = $conn->getTableNameComponents( $table );

		DB::_fillTablesList( $conn );

		//	exact match
		foreach( $dalTables[$conn->connId] as $t )
		{
			if( $t["schema"] == $tableName["schema"] && $t["name"] == $tableName["table"] )
				return $t;
		}

		//	case-insensitive
		$tableName["schema"] = strtoupper( $tableName["schema"] );
		$tableName["table"] = strtoupper( $tableName["table"] );

		foreach( $dalTables[$conn->connId] as $t )
		{
			if( strtoupper( $t["schema"] ) == $tableName["schema"] && strtoupper( $t["name"] ) == $tableName["table"] )
				return $t;
		}
		return null;
	}

	/**
	 * 	Get list of table field names and types
	 *	Check tables stored in the project first, then fetch it from the database.
	 *
	 */
	public static function _getTableInfo($table, $connId = null )
	{
		global $dal_info, $tableinfo_cache, $cman;
		if( !$connId )
			$connId = DB::CurrentConnectionId();

		//	prepare cache
		if( !isset($tableinfo_cache[ $connId ] ) )
			$tableinfo_cache[ $connId ] = array();

		$tableInfo = array();


		$tableDescriptor = DB::_findDalTable( $table, $cman->byId( $connId ) );

		if ( $tableDescriptor )
		{
			importTableInfo( $tableDescriptor["varname"] );

			$tableInfo["fields"] = $dal_info[ $tableDescriptor["varname"] ];

			if( $tableDescriptor["schema"] )
				$tableInfo["fullName"] = $tableDescriptor["schema"] . "." . $tableDescriptor["name"];
			else
				$tableInfo["fullName"] = $tableDescriptor["name"];
		}
		else
		{
			//	check cache first
			if( isset($tableinfo_cache[ $connId ][ $table ] ) )
				return $tableinfo_cache[ $connId ][ $table ];

			//	fetch table info from the database
			$helpSql = "select * from " . DB::CurrentConnection()->addTableWrappers( $table ) . " where 1=0";

			$tableInfo["fullName"] = $table;
			$tableInfo["fields"] = array();

			// in case getFieldsList throws error
			$tableinfo_cache[ $connId ][ $table ] = false;

			$fieldList = DB::CurrentConnection()->getFieldsList($helpSql);
			foreach ($fieldList as $f )
			{
				$tableInfo["fields"][ $f["fieldname"] ] = array( "type" => $f["type"], "name" => $f["fieldname"] );
			}
			$tableinfo_cache[ $connId ][ $table ] = $tableInfo;
		}

		return $tableInfo;
	}


	protected static function _fillTablesList( $conn )
	{
		global $dalTables;
		if( !$conn )
			$conn = DB::CurrentConnection();
		if( $dalTables[ $conn->connId ] )
			return;
		$dalTables[ $conn->connId ] = array();
		if( "Tables" == $conn->connId )
		{
			$dalTables[$conn->connId][] = array("name" => "accessories", "varname" => "Tables__accessories", "altvarname" => "accessories", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "account_classification", "varname" => "Tables__account_classification", "altvarname" => "account_classification", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "account_general_classification", "varname" => "Tables__account_general_classification", "altvarname" => "account_general_classification", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "account_type", "varname" => "Tables__account_type", "altvarname" => "account_type", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "accounting_period", "varname" => "Tables__accounting_period", "altvarname" => "accounting_period", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "aging_ap", "varname" => "Tables__aging_ap", "altvarname" => "aging_ap", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "aging_ar", "varname" => "Tables__aging_ar", "altvarname" => "aging_ar", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "approval_alerts", "varname" => "Tables__approval_alerts", "altvarname" => "approval_alerts", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "approver_list", "varname" => "Tables__approver_list", "altvarname" => "approver_list", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "area", "varname" => "Tables__area", "altvarname" => "area", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "asset_accountability", "varname" => "Tables__asset_accountability", "altvarname" => "asset_accountability", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "asset_accountability_detail", "varname" => "Tables__asset_accountability_detail", "altvarname" => "asset_accountability_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "attendance", "varname" => "Tables__attendance", "altvarname" => "attendance", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "attendance_report", "varname" => "Tables__attendance_report", "altvarname" => "attendance_report", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "backload", "varname" => "Tables__backload", "altvarname" => "backload", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "backload_detail", "varname" => "Tables__backload_detail", "altvarname" => "backload_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bal_cashpayment", "varname" => "Tables__bal_cashpayment", "altvarname" => "bal_cashpayment", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bal_cashreceipt", "varname" => "Tables__bal_cashreceipt", "altvarname" => "bal_cashreceipt", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bal_invoice_credit_memo", "varname" => "Tables__bal_invoice_credit_memo", "altvarname" => "bal_invoice_credit_memo", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bal_po", "varname" => "Tables__bal_po", "altvarname" => "bal_po", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bal_purchase_debit_memo", "varname" => "Tables__bal_purchase_debit_memo", "altvarname" => "bal_purchase_debit_memo", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bal_purchase_head_debit_memo", "varname" => "Tables__bal_purchase_head_debit_memo", "altvarname" => "bal_purchase_head_debit_memo", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bal_purchase_order_detail", "varname" => "Tables__bal_purchase_order_detail", "altvarname" => "bal_purchase_order_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bal_sales_credit_memo", "varname" => "Tables__bal_sales_credit_memo", "altvarname" => "bal_sales_credit_memo", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bal_sales_detail", "varname" => "Tables__bal_sales_detail", "altvarname" => "bal_sales_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bal_sales_order_detail", "varname" => "Tables__bal_sales_order_detail", "altvarname" => "bal_sales_order_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bal_total_po_detail", "varname" => "Tables__bal_total_po_detail", "altvarname" => "bal_total_po_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bank", "varname" => "Tables__bank", "altvarname" => "bank", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bank_collection", "varname" => "Tables__bank_collection", "altvarname" => "bank_collection", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bank_disbursement", "varname" => "Tables__bank_disbursement", "altvarname" => "bank_disbursement", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "bank_list", "varname" => "Tables__bank_list", "altvarname" => "bank_list", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "barangay", "varname" => "Tables__barangay", "altvarname" => "barangay", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "basic_system_audit", "varname" => "Tables__basic_system_audit", "altvarname" => "basic_system_audit", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "beg_bal", "varname" => "Tables__beg_bal", "altvarname" => "beg_bal", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "beg_baldetail", "varname" => "Tables__beg_baldetail", "altvarname" => "beg_baldetail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "branch_sales", "varname" => "Tables__branch_sales", "altvarname" => "branch_sales", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "branch_sales_detail", "varname" => "Tables__branch_sales_detail", "altvarname" => "branch_sales_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "budget", "varname" => "Tables__budget", "altvarname" => "budget", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "budget_detail", "varname" => "Tables__budget_detail", "altvarname" => "budget_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "business_partner", "varname" => "Tables__business_partner", "altvarname" => "business_partner", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "business_partner_contacts", "varname" => "Tables__business_partner_contacts", "altvarname" => "business_partner_contacts", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "business_partner_group", "varname" => "Tables__business_partner_group", "altvarname" => "business_partner_group", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "calls", "varname" => "Tables__calls", "altvarname" => "calls", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "campaign", "varname" => "Tables__campaign", "altvarname" => "campaign", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "campaign_type", "varname" => "Tables__campaign_type", "altvarname" => "campaign_type", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "case", "varname" => "Tables__tblcase", "altvarname" => "tblcase", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "case_solutions", "varname" => "Tables__case_solutions", "altvarname" => "case_solutions", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "case_type", "varname" => "Tables__case_type", "altvarname" => "case_type", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "cash_disbursement_detail2", "varname" => "Tables__cash_disbursement_detail2", "altvarname" => "cash_disbursement_detail2", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "cash_disbursment", "varname" => "Tables__cash_disbursment", "altvarname" => "cash_disbursment", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "cash_disbursment_detail", "varname" => "Tables__cash_disbursment_detail", "altvarname" => "cash_disbursment_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "cash_pullout", "varname" => "Tables__cash_pullout", "altvarname" => "cash_pullout", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "cash_pullout_detail", "varname" => "Tables__cash_pullout_detail", "altvarname" => "cash_pullout_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "cash_receipt", "varname" => "Tables__cash_receipt", "altvarname" => "cash_receipt", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "cash_receipt_cost_detail", "varname" => "Tables__cash_receipt_cost_detail", "altvarname" => "cash_receipt_cost_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "cash_receipt_detail", "varname" => "Tables__cash_receipt_detail", "altvarname" => "cash_receipt_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "cash_receipt_detail2", "varname" => "Tables__cash_receipt_detail2", "altvarname" => "cash_receipt_detail2", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "center_replenishment", "varname" => "Tables__center_replenishment", "altvarname" => "center_replenishment", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "center_replenishment_detail", "varname" => "Tables__center_replenishment_detail", "altvarname" => "center_replenishment_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "chart_of_accounts", "varname" => "Tables__chart_of_accounts", "altvarname" => "chart_of_accounts", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "check_book_registry", "varname" => "Tables__check_book_registry", "altvarname" => "check_book_registry", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "check_book_registry_detail", "varname" => "Tables__check_book_registry_detail", "altvarname" => "check_book_registry_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "check_formats", "varname" => "Tables__check_formats", "altvarname" => "check_formats", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "checklist", "varname" => "Tables__checklist", "altvarname" => "checklist", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "city", "varname" => "Tables__city", "altvarname" => "city", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "company_info", "varname" => "Tables__company_info", "altvarname" => "company_info", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "competitor", "varname" => "Tables__competitor", "altvarname" => "competitor", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "cost_center", "varname" => "Tables__cost_center", "altvarname" => "cost_center", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "credit_memo", "varname" => "Tables__credit_memo", "altvarname" => "credit_memo", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "credit_memo_detail", "varname" => "Tables__credit_memo_detail", "altvarname" => "credit_memo_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "credit_memo_detail_cgs", "varname" => "Tables__credit_memo_detail_cgs", "altvarname" => "credit_memo_detail_cgs", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "credit_terms", "varname" => "Tables__credit_terms", "altvarname" => "credit_terms", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "currency", "varname" => "Tables__currency", "altvarname" => "currency", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "currency_exchange_rate", "varname" => "Tables__currency_exchange_rate", "altvarname" => "currency_exchange_rate", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "customer_shipping_address", "varname" => "Tables__customer_shipping_address", "altvarname" => "customer_shipping_address", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "debit_memo", "varname" => "Tables__debit_memo", "altvarname" => "debit_memo", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "debit_memo_detail", "varname" => "Tables__debit_memo_detail", "altvarname" => "debit_memo_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "demand_plan", "varname" => "Tables__demand_plan", "altvarname" => "demand_plan", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "department", "varname" => "Tables__department", "altvarname" => "department", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "deposit", "varname" => "Tables__deposit", "altvarname" => "deposit", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "deposit_detail", "varname" => "Tables__deposit_detail", "altvarname" => "deposit_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "discounts", "varname" => "Tables__discounts", "altvarname" => "discounts", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "disposed", "varname" => "Tables__disposed", "altvarname" => "disposed", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "disposed_detail", "varname" => "Tables__disposed_detail", "altvarname" => "disposed_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "disposed_inv", "varname" => "Tables__disposed_inv", "altvarname" => "disposed_inv", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "disposed_inv_detail", "varname" => "Tables__disposed_inv_detail", "altvarname" => "disposed_inv_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "document", "varname" => "Tables__document", "altvarname" => "document", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "document_type", "varname" => "Tables__document_type", "altvarname" => "document_type", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "dr", "varname" => "Tables__dr", "altvarname" => "dr", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "dr_detail", "varname" => "Tables__dr_detail", "altvarname" => "dr_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "email", "varname" => "Tables__email", "altvarname" => "email", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "employees", "varname" => "Tables__employees", "altvarname" => "employees", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "estimate", "varname" => "Tables__estimate", "altvarname" => "estimate", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "estimate_cost", "varname" => "Tables__estimate_cost", "altvarname" => "estimate_cost", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "estimate_detail", "varname" => "Tables__estimate_detail", "altvarname" => "estimate_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "ewt", "varname" => "Tables__ewt", "altvarname" => "ewt", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "excise_tax", "varname" => "Tables__excise_tax", "altvarname" => "excise_tax", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "expense", "varname" => "Tables__expense", "altvarname" => "expense", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "financial_statement", "varname" => "Tables__financial_statement", "altvarname" => "financial_statement", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "fiscal_year", "varname" => "Tables__fiscal_year", "altvarname" => "fiscal_year", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "fixed_asset_issue", "varname" => "Tables__fixed_asset_issue", "altvarname" => "fixed_asset_issue", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "fixed_asset_issue_detail", "varname" => "Tables__fixed_asset_issue_detail", "altvarname" => "fixed_asset_issue_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "fixed_asset_return_detail", "varname" => "Tables__fixed_asset_return_detail", "altvarname" => "fixed_asset_return_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "fixed_asset_time_usage", "varname" => "Tables__fixed_asset_time_usage", "altvarname" => "fixed_asset_time_usage", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "fixed_asset_time_usage_detail", "varname" => "Tables__fixed_asset_time_usage_detail", "altvarname" => "fixed_asset_time_usage_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "fixed_assets_return", "varname" => "Tables__fixed_assets_return", "altvarname" => "fixed_assets_return", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "foodbasket_uggroups", "varname" => "Tables__foodbasket_uggroups", "altvarname" => "foodbasket_uggroups", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "foodbasket_ugmembers", "varname" => "Tables__foodbasket_ugmembers", "altvarname" => "foodbasket_ugmembers", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "foodbasket_ugrights", "varname" => "Tables__foodbasket_ugrights", "altvarname" => "foodbasket_ugrights", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "for_submissions", "varname" => "Tables__for_submissions", "altvarname" => "for_submissions", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "gas_type", "varname" => "Tables__gas_type", "altvarname" => "gas_type", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "general_journal", "varname" => "Tables__general_journal", "altvarname" => "general_journal", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "general_journal_detail", "varname" => "Tables__general_journal_detail", "altvarname" => "general_journal_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "import_inv_adj", "varname" => "Tables__import_inv_adj", "altvarname" => "import_inv_adj", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "import_inventory_transfer", "varname" => "Tables__import_inventory_transfer", "altvarname" => "import_inventory_transfer", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "import_sales", "varname" => "Tables__import_sales", "altvarname" => "import_sales", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "input_vat_allocation", "varname" => "Tables__input_vat_allocation", "altvarname" => "input_vat_allocation", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inv_adj_detail", "varname" => "Tables__inv_adj_detail", "altvarname" => "inv_adj_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inv_bal", "varname" => "Tables__inv_bal", "altvarname" => "inv_bal", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inv_balance", "varname" => "Tables__inv_balance", "altvarname" => "inv_balance", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inv_fifo", "varname" => "Tables__inv_fifo", "altvarname" => "inv_fifo", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inv_inv_production_fg", "varname" => "Tables__inv_inv_production_fg", "altvarname" => "inv_inv_production_fg", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inv_inv_production_rw", "varname" => "Tables__inv_inv_production_rw", "altvarname" => "inv_inv_production_rw", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inv_inv_purchase", "varname" => "Tables__inv_inv_purchase", "altvarname" => "inv_inv_purchase", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inv_ledger", "varname" => "Tables__inv_ledger", "altvarname" => "inv_ledger", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inventory_adj", "varname" => "Tables__inventory_adj", "altvarname" => "inventory_adj", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inventory_adjustment_type", "varname" => "Tables__inventory_adjustment_type", "altvarname" => "inventory_adjustment_type", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inventory_tranfer", "varname" => "Tables__inventory_tranfer", "altvarname" => "inventory_tranfer", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inventory_tranfer_request", "varname" => "Tables__inventory_tranfer_request", "altvarname" => "inventory_tranfer_request", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inventory_transfer__request_detail", "varname" => "Tables__inventory_transfer__request_detail", "altvarname" => "inventory_transfer__request_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inventory_transfer_cost_detail", "varname" => "Tables__inventory_transfer_cost_detail", "altvarname" => "inventory_transfer_cost_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "inventory_transfer_detail", "varname" => "Tables__inventory_transfer_detail", "altvarname" => "inventory_transfer_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "issue", "varname" => "Tables__issue", "altvarname" => "issue", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "issue_component_imp", "varname" => "Tables__issue_component_imp", "altvarname" => "issue_component_imp", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "issue_components", "varname" => "Tables__issue_components", "altvarname" => "issue_components", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "issue_components_detail", "varname" => "Tables__issue_components_detail", "altvarname" => "issue_components_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "issue_detail", "varname" => "Tables__issue_detail", "altvarname" => "issue_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "item", "varname" => "Tables__item", "altvarname" => "item", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "item_bill_of_materials", "varname" => "Tables__item_bill_of_materials", "altvarname" => "item_bill_of_materials", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "item_bom_type", "varname" => "Tables__item_bom_type", "altvarname" => "item_bom_type", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "item_cost_center_detail", "varname" => "Tables__item_cost_center_detail", "altvarname" => "item_cost_center_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "item_cost_recalculation", "varname" => "Tables__item_cost_recalculation", "altvarname" => "item_cost_recalculation", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "item_expiration", "varname" => "Tables__item_expiration", "altvarname" => "item_expiration", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "item_group", "varname" => "Tables__item_group", "altvarname" => "item_group", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "item_group_price_per_category", "varname" => "Tables__item_group_price_per_category", "altvarname" => "item_group_price_per_category", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "item_price_per_category", "varname" => "Tables__item_price_per_category", "altvarname" => "item_price_per_category", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "item_production_phase", "varname" => "Tables__item_production_phase", "altvarname" => "item_production_phase", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "item_uom_conversion", "varname" => "Tables__item_uom_conversion", "altvarname" => "item_uom_conversion", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_cash_disbursement", "varname" => "Tables__journal_entry_cash_disbursement", "altvarname" => "journal_entry_cash_disbursement", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_cash_receipt", "varname" => "Tables__journal_entry_cash_receipt", "altvarname" => "journal_entry_cash_receipt", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_center_replenishment", "varname" => "Tables__journal_entry_center_replenishment", "altvarname" => "journal_entry_center_replenishment", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_credit_memo", "varname" => "Tables__journal_entry_credit_memo", "altvarname" => "journal_entry_credit_memo", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_debit_memo", "varname" => "Tables__journal_entry_debit_memo", "altvarname" => "journal_entry_debit_memo", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_deposit", "varname" => "Tables__journal_entry_deposit", "altvarname" => "journal_entry_deposit", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_depreciation", "varname" => "Tables__journal_entry_depreciation", "altvarname" => "journal_entry_depreciation", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_general_journal", "varname" => "Tables__journal_entry_general_journal", "altvarname" => "journal_entry_general_journal", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_inv_transfer", "varname" => "Tables__journal_entry_inv_transfer", "altvarname" => "journal_entry_inv_transfer", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_inventory_adjustment", "varname" => "Tables__journal_entry_inventory_adjustment", "altvarname" => "journal_entry_inventory_adjustment", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_liquidation", "varname" => "Tables__journal_entry_liquidation", "altvarname" => "journal_entry_liquidation", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_production", "varname" => "Tables__journal_entry_production", "altvarname" => "journal_entry_production", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_project_replenishment", "varname" => "Tables__journal_entry_project_replenishment", "altvarname" => "journal_entry_project_replenishment", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_purchases", "varname" => "Tables__journal_entry_purchases", "altvarname" => "journal_entry_purchases", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_replenishment", "varname" => "Tables__journal_entry_replenishment", "altvarname" => "journal_entry_replenishment", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_sales", "varname" => "Tables__journal_entry_sales", "altvarname" => "journal_entry_sales", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_entry_service_revenue", "varname" => "Tables__journal_entry_service_revenue", "altvarname" => "journal_entry_service_revenue", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "journal_ppe_disposal", "varname" => "Tables__journal_ppe_disposal", "altvarname" => "journal_ppe_disposal", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "liquidation", "varname" => "Tables__liquidation", "altvarname" => "liquidation", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "liquidation_detail", "varname" => "Tables__liquidation_detail", "altvarname" => "liquidation_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "loan", "varname" => "Tables__loan", "altvarname" => "loan", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "loan__detail", "varname" => "Tables__loan__detail", "altvarname" => "loan__detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "location", "varname" => "Tables__location", "altvarname" => "location", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "lot", "varname" => "Tables__lot", "altvarname" => "lot", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "loyalty_points", "varname" => "Tables__loyalty_points", "altvarname" => "loyalty_points", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "marketing_colateral", "varname" => "Tables__marketing_colateral", "altvarname" => "marketing_colateral", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "marketing_job_order", "varname" => "Tables__marketing_job_order", "altvarname" => "marketing_job_order", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "marketing_request_form", "varname" => "Tables__marketing_request_form", "altvarname" => "marketing_request_form", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "meetings", "varname" => "Tables__meetings", "altvarname" => "meetings", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "memorized_transactions", "varname" => "Tables__memorized_transactions", "altvarname" => "memorized_transactions", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "month_values", "varname" => "Tables__month_values", "altvarname" => "month_values", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "new_ticket", "varname" => "Tables__new_ticket", "altvarname" => "new_ticket", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "new_user", "varname" => "Tables__new_user", "altvarname" => "new_user", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "open_backload", "varname" => "Tables__open_backload", "altvarname" => "open_backload", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "open_dr_for_invoice", "varname" => "Tables__open_dr_for_invoice", "altvarname" => "open_dr_for_invoice", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "open_estimates", "varname" => "Tables__open_estimates", "altvarname" => "open_estimates", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "open_po", "varname" => "Tables__open_po", "altvarname" => "open_po", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "open_pr", "varname" => "Tables__open_pr", "altvarname" => "open_pr", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "open_production_request", "varname" => "Tables__open_production_request", "altvarname" => "open_production_request", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "open_qc", "varname" => "Tables__open_qc", "altvarname" => "open_qc", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "open_quote", "varname" => "Tables__open_quote", "altvarname" => "open_quote", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "open_receiving", "varname" => "Tables__open_receiving", "altvarname" => "open_receiving", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "open_request_for_payment", "varname" => "Tables__open_request_for_payment", "altvarname" => "open_request_for_payment", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "open_rfp_to_apv", "varname" => "Tables__open_rfp_to_apv", "altvarname" => "open_rfp_to_apv", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "open_so", "varname" => "Tables__open_so", "altvarname" => "open_so", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "open_so_for_invoice", "varname" => "Tables__open_so_for_invoice", "altvarname" => "open_so_for_invoice", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "open_so_for_project_management", "varname" => "Tables__open_so_for_project_management", "altvarname" => "open_so_for_project_management", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "opportunity", "varname" => "Tables__opportunity", "altvarname" => "opportunity", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "origin_fa", "varname" => "Tables__origin_fa", "altvarname" => "origin_fa", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "origin_serialized_supply", "varname" => "Tables__origin_serialized_supply", "altvarname" => "origin_serialized_supply", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "packing_list", "varname" => "Tables__packing_list", "altvarname" => "packing_list", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "packing_list_detail", "varname" => "Tables__packing_list_detail", "altvarname" => "packing_list_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "payment_method", "varname" => "Tables__payment_method", "altvarname" => "payment_method", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "payroll_item", "varname" => "Tables__payroll_item", "altvarname" => "payroll_item", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "pdc_registry", "varname" => "Tables__pdc_registry", "altvarname" => "pdc_registry", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "pending_items_reminders", "varname" => "Tables__pending_items_reminders", "altvarname" => "pending_items_reminders", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "petty_cash", "varname" => "Tables__petty_cash", "altvarname" => "petty_cash", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "petty_cash_detail", "varname" => "Tables__petty_cash_detail", "altvarname" => "petty_cash_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "phase", "varname" => "Tables__phase", "altvarname" => "phase", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "phase_bill_of_materials", "varname" => "Tables__phase_bill_of_materials", "altvarname" => "phase_bill_of_materials", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "phase_billing", "varname" => "Tables__phase_billing", "altvarname" => "phase_billing", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "picklist", "varname" => "Tables__picklist", "altvarname" => "picklist", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "picklist_detail", "varname" => "Tables__picklist_detail", "altvarname" => "picklist_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "picklist_detail_done", "varname" => "Tables__picklist_detail_done", "altvarname" => "picklist_detail_done", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "position", "varname" => "Tables__position", "altvarname" => "position", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "ppe_list", "varname" => "Tables__ppe_list", "altvarname" => "ppe_list", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "process_payroll", "varname" => "Tables__process_payroll", "altvarname" => "process_payroll", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "process_payroll_detail", "varname" => "Tables__process_payroll_detail", "altvarname" => "process_payroll_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "prod_vs_purchase", "varname" => "Tables__prod_vs_purchase", "altvarname" => "prod_vs_purchase", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "production", "varname" => "Tables__production", "altvarname" => "production", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "production _imp", "varname" => "Tables__production__imp", "altvarname" => "production__imp", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "production_bill_of_material", "varname" => "Tables__production_bill_of_material", "altvarname" => "production_bill_of_material", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "production_bom", "varname" => "Tables__production_bom", "altvarname" => "production_bom", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "production_cost_detail", "varname" => "Tables__production_cost_detail", "altvarname" => "production_cost_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "production_detail", "varname" => "Tables__production_detail", "altvarname" => "production_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "production_form", "varname" => "Tables__production_form", "altvarname" => "production_form", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "production_itemized_detail", "varname" => "Tables__production_itemized_detail", "altvarname" => "production_itemized_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "production_request", "varname" => "Tables__production_request", "altvarname" => "production_request", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "production_request_bom", "varname" => "Tables__production_request_bom", "altvarname" => "production_request_bom", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "production_request_detail", "varname" => "Tables__production_request_detail", "altvarname" => "production_request_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "production_stage_cost_detail", "varname" => "Tables__production_stage_cost_detail", "altvarname" => "production_stage_cost_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "production_stage_status_cost_detail", "varname" => "Tables__production_stage_status_cost_detail", "altvarname" => "production_stage_status_cost_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "production_stage_status_detail", "varname" => "Tables__production_stage_status_detail", "altvarname" => "production_stage_status_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "project_bill_of_material", "varname" => "Tables__project_bill_of_material", "altvarname" => "project_bill_of_material", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "project_progress_billing", "varname" => "Tables__project_progress_billing", "altvarname" => "project_progress_billing", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "project_replenishment", "varname" => "Tables__project_replenishment", "altvarname" => "project_replenishment", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "project_replenishment_detail", "varname" => "Tables__project_replenishment_detail", "altvarname" => "project_replenishment_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "projects", "varname" => "Tables__projects", "altvarname" => "projects", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "province", "varname" => "Tables__province", "altvarname" => "province", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "purchase", "varname" => "Tables__purchase", "altvarname" => "purchase", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "purchase_detail", "varname" => "Tables__purchase_detail", "altvarname" => "purchase_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "purchase_order", "varname" => "Tables__purchase_order", "altvarname" => "purchase_order", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "purchase_order_detail", "varname" => "Tables__purchase_order_detail", "altvarname" => "purchase_order_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "purchase_order_receiving", "varname" => "Tables__purchase_order_receiving", "altvarname" => "purchase_order_receiving", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "purchase_order_receiving_detail", "varname" => "Tables__purchase_order_receiving_detail", "altvarname" => "purchase_order_receiving_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "purchase_report_per_item", "varname" => "Tables__purchase_report_per_item", "altvarname" => "purchase_report_per_item", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "purchase_request", "varname" => "Tables__purchase_request", "altvarname" => "purchase_request", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "purchase_request_detail", "varname" => "Tables__purchase_request_detail", "altvarname" => "purchase_request_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "qc_inspection", "varname" => "Tables__qc_inspection", "altvarname" => "qc_inspection", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "quality_control", "varname" => "Tables__quality_control", "altvarname" => "quality_control", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "quality_control_detail", "varname" => "Tables__quality_control_detail", "altvarname" => "quality_control_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "quotation", "varname" => "Tables__quotation", "altvarname" => "quotation", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "quotation_detail", "varname" => "Tables__quotation_detail", "altvarname" => "quotation_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "rdo", "varname" => "Tables__rdo", "altvarname" => "rdo", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "region", "varname" => "Tables__region", "altvarname" => "region", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "reimbursement", "varname" => "Tables__reimbursement", "altvarname" => "reimbursement", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "reimbursement_detail", "varname" => "Tables__reimbursement_detail", "altvarname" => "reimbursement_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "repair_list", "varname" => "Tables__repair_list", "altvarname" => "repair_list", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "repairs_and_maintenance", "varname" => "Tables__repairs_and_maintenance", "altvarname" => "repairs_and_maintenance", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "repairs_and_maintenance_detail", "varname" => "Tables__repairs_and_maintenance_detail", "altvarname" => "repairs_and_maintenance_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "repairs_particular", "varname" => "Tables__repairs_particular", "altvarname" => "repairs_particular", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "report_center", "varname" => "Tables__report_center", "altvarname" => "report_center", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "request_for_payment", "varname" => "Tables__request_for_payment", "altvarname" => "request_for_payment", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "request_for_payment_detail", "varname" => "Tables__request_for_payment_detail", "altvarname" => "request_for_payment_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "request_for_payment_imp", "varname" => "Tables__request_for_payment_imp", "altvarname" => "request_for_payment_imp", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "request_for_quote", "varname" => "Tables__request_for_quote", "altvarname" => "request_for_quote", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "request_for_quoter_detail", "varname" => "Tables__request_for_quoter_detail", "altvarname" => "request_for_quoter_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "return", "varname" => "Tables__tblreturn", "altvarname" => "tblreturn", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "return_components", "varname" => "Tables__return_components", "altvarname" => "return_components", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "return_components_detail", "varname" => "Tables__return_components_detail", "altvarname" => "return_components_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "return_detail", "varname" => "Tables__return_detail", "altvarname" => "return_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "rfp", "varname" => "Tables__rfp", "altvarname" => "rfp", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sale_assemble_detail", "varname" => "Tables__sale_assemble_detail", "altvarname" => "sale_assemble_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sale_cost_detail", "varname" => "Tables__sale_cost_detail", "altvarname" => "sale_cost_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sales", "varname" => "Tables__sales", "altvarname" => "sales", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sales_detail", "varname" => "Tables__sales_detail", "altvarname" => "sales_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sales_order", "varname" => "Tables__sales_order", "altvarname" => "sales_order", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sales_order_detail", "varname" => "Tables__sales_order_detail", "altvarname" => "sales_order_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sales_order_detail_billing_dates", "varname" => "Tables__sales_order_detail_billing_dates", "altvarname" => "sales_order_detail_billing_dates", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sales_order_materials_detail", "varname" => "Tables__sales_order_materials_detail", "altvarname" => "sales_order_materials_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sales_order_progress_billing_detail", "varname" => "Tables__sales_order_progress_billing_detail", "altvarname" => "sales_order_progress_billing_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sales_report_per_item", "varname" => "Tables__sales_report_per_item", "altvarname" => "sales_report_per_item", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "salesapp", "varname" => "Tables__salesapp", "altvarname" => "salesapp", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "search_beg_bal", "varname" => "Tables__search_beg_bal", "altvarname" => "search_beg_bal", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "search_cash_disbursement", "varname" => "Tables__search_cash_disbursement", "altvarname" => "search_cash_disbursement", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "search_cash_receipt", "varname" => "Tables__search_cash_receipt", "altvarname" => "search_cash_receipt", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "search_general_journal", "varname" => "Tables__search_general_journal", "altvarname" => "search_general_journal", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "search_purchase_detail", "varname" => "Tables__search_purchase_detail", "altvarname" => "search_purchase_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "serial_no", "varname" => "Tables__serial_no", "altvarname" => "serial_no", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "serialized_supply_list", "varname" => "Tables__serialized_supply_list", "altvarname" => "serialized_supply_list", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "service_center", "varname" => "Tables__service_center", "altvarname" => "service_center", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "service_report", "varname" => "Tables__service_report", "altvarname" => "service_report", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "service_report_detail", "varname" => "Tables__service_report_detail", "altvarname" => "service_report_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "service_visit", "varname" => "Tables__service_visit", "altvarname" => "service_visit", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "shipping_terms", "varname" => "Tables__shipping_terms", "altvarname" => "shipping_terms", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "so_imp", "varname" => "Tables__so_imp", "altvarname" => "so_imp", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "spoilage", "varname" => "Tables__spoilage", "altvarname" => "spoilage", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "spoilage_detail", "varname" => "Tables__spoilage_detail", "altvarname" => "spoilage_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "spoilage_detail2", "varname" => "Tables__spoilage_detail2", "altvarname" => "spoilage_detail2", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "stage", "varname" => "Tables__stage", "altvarname" => "stage", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "submission_details", "varname" => "Tables__submission_details", "altvarname" => "submission_details", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sum_cr", "varname" => "Tables__sum_cr", "altvarname" => "sum_cr", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sum_creditmemo", "varname" => "Tables__sum_creditmemo", "altvarname" => "sum_creditmemo", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sum_cvoucher", "varname" => "Tables__sum_cvoucher", "altvarname" => "sum_cvoucher", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sum_debitmemo", "varname" => "Tables__sum_debitmemo", "altvarname" => "sum_debitmemo", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "sum_dr_qty", "varname" => "Tables__sum_dr_qty", "altvarname" => "sum_dr_qty", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "system_user", "varname" => "Tables__system_user", "altvarname" => "system_user", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "tara_sheet", "varname" => "Tables__tara_sheet", "altvarname" => "tara_sheet", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "tara_sheet_detail", "varname" => "Tables__tara_sheet_detail", "altvarname" => "tara_sheet_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "tasks", "varname" => "Tables__tasks", "altvarname" => "tasks", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "tax_person_status", "varname" => "Tables__tax_person_status", "altvarname" => "tax_person_status", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "tax_status", "varname" => "Tables__tax_status", "altvarname" => "tax_status", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "tbtransact", "varname" => "Tables__tbtransact", "altvarname" => "tbtransact", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "telex", "varname" => "Tables__telex", "altvarname" => "telex", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "tranasactions_for_approval", "varname" => "Tables__tranasactions_for_approval", "altvarname" => "tranasactions_for_approval", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "transmittal_slip", "varname" => "Tables__transmittal_slip", "altvarname" => "transmittal_slip", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "transmittal_slip_detail", "varname" => "Tables__transmittal_slip_detail", "altvarname" => "transmittal_slip_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "trip_detail", "varname" => "Tables__trip_detail", "altvarname" => "trip_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "trip_ticket", "varname" => "Tables__trip_ticket", "altvarname" => "trip_ticket", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "uom", "varname" => "Tables__uom", "altvarname" => "uom", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "user_roles", "varname" => "Tables__user_roles", "altvarname" => "user_roles", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "vat_atc", "varname" => "Tables__vat_atc", "altvarname" => "vat_atc", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "vat_relief_importation", "varname" => "Tables__vat_relief_importation", "altvarname" => "vat_relief_importation", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "vat_relief_purchases", "varname" => "Tables__vat_relief_purchases", "altvarname" => "vat_relief_purchases", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "vat_relief_sales", "varname" => "Tables__vat_relief_sales", "altvarname" => "vat_relief_sales", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "vat_type", "varname" => "Tables__vat_type", "altvarname" => "vat_type", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "vehicle", "varname" => "Tables__vehicle", "altvarname" => "vehicle", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "waranty_detail", "varname" => "Tables__waranty_detail", "altvarname" => "waranty_detail", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
			$dalTables[$conn->connId][] = array("name" => "years", "varname" => "Tables__years", "altvarname" => "years", "connId" => "Tables", "schema" => "", "connName" => "nboxsy5_erp_foodbasket at localhost");
		}
	}


	public static function PrepareSQL($sql)
	{
		$args = func_get_args();

		$conn = DB::CurrentConnection();

		$tokens = DB::scanTokenString($sql);

		$replacements = array();
		// build array of replacements in this format:
		//	"offset" => position in the string where replacement should be done
		//  "len" => length of original substring to cut out
		//  "insert" => string to insert in place of cut out

		foreach ($tokens["matches"] as $i => $match) {
			$offset = $tokens["offsets"][$i];
			$token = $tokens["tokens"][$i];

			$repl = array(
				"offset" => $offset,
				"len" => strlen($match)
			);

			$val = "";
			if (is_numeric($token) && count( $args ) > $token) {
				$val = $args[(int)$token];
			} else {
				$val = RunnerContext::getValue($token);
			}

			/**
			 * Don't ever dare to alter this code!
			 * Everything outside quotes must be converted to number to avoid SQL injection
			 */
			if ($conn->positionQuoted($sql, $offset))
				$repl["insert"] = $conn->addSlashes($val);
			else
				$repl["insert"] = DB::prepareNumberValue($val);

			$replacements[] = $repl;
		}

		//	do replacements
		return RunnerContext::doReplacements( $sql, $replacements );
	}

	/**
	 *	@return Array
	 */
	public static function readSQLTokens( $sql )
	{
		$arr = DB::scanTokenString( $sql );
		return $arr["tokens"];
	}

	/**
	 *	@return Array
	 */
	public static function readMasterTokens( $sql )
	{
		$masterTokens = array();

		$allTokens = DB::readSQLTokens( $sql );
		foreach ( $allTokens as $key => $token )
		{
			$dotPos = strpos(  $token, "." );
			if( $dotPos !== FALSE && strtolower( substr( $token, 0, $dotPos ) ) == "master")
			{
				$masterTokens[] = $token;
			}
		}

		return $masterTokens;
	}

	/**
	 *	Scans SQL string, finds all tokens. Returns three arrays - 'tokens', 'matches' and 'offsets'
	 *  Offsets are positions of corresponding 'matches' items in the string
	 *  Example:
	 *  insert into table values (':aaa', :old.bbb, ':{master.order date}')
	 *  tokens: ["aaa", "old.bbb", "master.order date"]
	 *  matches: [":aaa", ":old.bbb", ":{master.order date}"]
	 *  offsets: [28, 35, 46]
	 *
	 *	Exceptions for tokens without {}
	 *	1. shouldn't start with number
	*		:62aaa
	 *	2. shouldn't follow letter
	 *		x:aaa
	 *	3. shouldn't follow :
	 *		::aaa
	 *
 	 *	@return Array [ "tokens" => Array, "matches" => Array, "offsets" => Array ]
	 */
	public static function scanTokenString($sql)
	{
		$tokens = array();
		$offsets = array();
		$matches = array();

		//	match aaa, old.bbb, master.order date from:
		//	insert into table values (':aaa', :old.bbb, ':{master.order date}')

		$pattern = '/(?:[^\w\:]|^)(\:([a-zA-Z_]{1}[\w\.]*))|\:\{(.*?)\}|(?:[^\w\:]|^)(\:([1-9]+[0-9]*))/';


		$result = findMatches($pattern, $sql);
		foreach ($result as $m) {
			if ($m["submatches"][0] != "") {
				// first variant, no {}
				$matches[] = $m["submatches"][0];
				$tokens[] = $m["submatches"][1];
				$offsets[] = $m["offset"] + strpos($m["match"], $m["submatches"][0]);
			} else if ($m["submatches"][2] != "") {
				// second variant, in {}
				$matches[] = $m["match"];
				$tokens[] = $m["submatches"][2];
				$offsets[] = $m["offset"];
			} else if ($m["submatches"][3] != "") {
				// third variant, numeric like (:1, ':2')
				$matches[] = $m["submatches"][3];
				$tokens[] = $m["submatches"][4];
				$offsets[] = $m["offset"] + strpos($m["match"], $m["submatches"][3]);
			}
		}

		return array("tokens" => $tokens, "matches" => $matches, "offsets" => $offsets);
	}

	public static function scanNewTokenString($sql)
	{
		$tokens = array();
		$offsets = array();
		$matches = array();

		//	match aaa, old.bbb, master.order date from:
		//	insert into table values (':aaa', :old.bbb, ':{master.order date}')

		$pattern = "/\\\${[^\\s\{\\}]+}/";


		$result = findMatches($pattern, $sql);
		foreach ($result as $m) {
			$match = $m["match"];
			if ( $match != "" ) {
				$matches[] = $match;
				$tokens[] = substr( $match, 2, strlen( $match ) - 3 );
				$offsets[] = $m["offset"];
			}
		}

		return array("tokens" => $tokens, "matches" => $matches, "offsets" => $offsets);
	}


	public static function prepareNumberValue( $value )
	{
		$strvalue = str_replace( ",", ".", (string)$value );
		if( is_numeric($strvalue) )
			return $strvalue;
		return 0;
	}

	public static function Lookup( $sql ) {
		$result = DB::Query( $sql );
		if( !$result ) {
			return null;
		}
		$data = $result->fetchNumeric();
		if( !$data ) {
			return null;
		}
		return $data[0];
	}
	
	public static function DBLookup( $sql ) {
		return DB::Lookup( $sql );
	}

}

?>