<?php
class ConnectionManager_Base
{
	/**
	 * Cached Connection objects
	 * @type Array
	 */
	protected $cache = array();

	/**
	 * Project connections data
	 * @type Array
	 */
	protected $_connectionsData;

	/**
	 * Project connections data
	 * @type Array
	 */
	protected $_connectionsIdByName = array();


	/**
	 * An array storing the correspondence between project
	 * datasource tables names and connections ids
	 * @type Array
	 */
	protected $_tablesConnectionIds;


	/**
	 * @constructor
	 */
	function __construct()
	{
		$this->_setConnectionsData();
		$this->_setTablesConnectionIds();
	}

	/**
	 * Get connection id by the table name
	 * @param String tName
	 * @return Connection
	 */
	public function getTableConnId( $tName )
	{
		return $this->_tablesConnectionIds[ $tName ];
	}


	/**
	 * Get connection object by the table name
	 * @param String tName
	 * @return Connection
	 */
	public function byTable( $tName )
	{
		$connId = $this->_tablesConnectionIds[ $tName ];
		if( !$connId )
			return $this->getDefault();
		return $this->byId( $connId );
	}

	/**
	 * Get connection object by the connection name
	 * @param String connName
	 * @return Connection
	 */
	public function byName( $connName )
	{
		$connId = $this->getIdByName( $connName );
		if( !$connId )
			return $this->getDefault();
		return $this->byId( $connId );
	}

	/**
	 * Get connection id by the connection name
	 * @param String connName
	 * @return String
	 */
	protected function getIdByName( $connName )
	{
		return $this->_connectionsIdByName[ $connName ];
	}

	/**
	 * Get connection object by the connection id
	 * @param String connId
	 * @return Connection
	 */
	public function byId( $connId )
	{
		if( !isset( $this->cache[ $connId ] ) ) {
			$conn = $this->getConnection( $connId );
			if( !$conn ) {
				global $restApis;
				$conn = $restApis->getConnection( $connId );
			}
			if( !$conn ) {
				$conn = $this->getDefault();
			}
			$this->cache[ $connId ] = $conn;
		}

		return $this->cache[ $connId ];
	}

	/**
	 * Get the default db connection class
	 * @return Connection
	 */
	public function getDefault()
	{
		return $this->byId( "Tables" );
	}

	/**
	 * Get the default db connection id
	 * @return String
	 */
	public function getDefaultConnId()
	{
		return "Tables";
	}



	/**
	 * Get the users table db connection
	 * @return Connection
	 */
	public function getForLogin() {
		return $this->byId( $this->getLoginConnId() );
	}

	public function getLoginConnId() {
		return "Tables";
	}


	/**
	 * Get the log table db connection
	 * @return Connection
	 */
	public function getForAudit()
	{
		return $this->byId( "Tables" );
	}

	/**
	 * Get the locking table db connection
	 * @return Connection
	 */
	public function getForLocking()
	{
		return $this->getDefault();
	}

	/**
	 * Get the 'ug_groups' table db connection
	 * @return Connection
	 */
	public function getForUserGroups() {
		return $this->byId( $this->getUserGroupsConnId() );
	}

	public function getUserGroupsConnId() {
		return "Tables";
	}

	/**
	 * Get the saved searches table db connection
	 * @return Connection
	 */
	public function getForSavedSearches()
	{
		return $this->byId( $this->getSavedSearchesConnId() );
	}
	
	/**
	 * Get the saved searches table db connection
	 * @return Connection
	 */
	public function getSavedSearchesConnId()
	{
		return "Tables";
	}	

	/**
	 * Get the webreports tables db connection
	 * @return Connection
	 */
	public function getForWebReports() 
	{
		return $this->byId( $this->getSavedSearchesConnId() );
	}

	/**
	 * Get the webreports tables db connection id
	 * @return String
	 */
	public function getWebReportsConnId() {
		return "Tables";
	}	
	
	/**
	 * @param String connId
	 * @return Connection
	 */
	protected function getConnection( $connId )
	{
		return false;
	}

	public function getConectionsIds()
	{
		$connectionsIds = array();
		foreach ($this->_connectionsData as $connId => $data) {
			$connectionsIds[] = $connId;
		}

		return $connectionsIds;
	}

	/**
	 * Set the data representing the project's
	 * db connection properties
	 */
	protected function _setConnectionsData()
	{
        return null;
	}

	/**
	 * Set the data representing the correspondence between
	 * the project's table names and db connections
	 */
	protected function _setTablesConnectionIds()
	{
		$connectionsIds = array();
		$connectionsIds["business_partner"] = "Tables";
		$connectionsIds["item"] = "Tables";
		$connectionsIds["credit_terms"] = "Tables";
		$connectionsIds["shipping_terms"] = "Tables";
		$connectionsIds["cash_receipt"] = "Tables";
		$connectionsIds["purchase"] = "Tables";
		$connectionsIds["cash_disbursment"] = "Tables";
		$connectionsIds["general_journal"] = "Tables";
		$connectionsIds["report_center"] = "Tables";
		$connectionsIds["Sales Report"] = "Tables";
		$connectionsIds["Disbursement Reports"] = "Tables";
		$connectionsIds["Financial Reports"] = "Tables";
		$connectionsIds["Purchase Reports"] = "Tables";
		$connectionsIds["Receivable Reports"] = "Tables";
		$connectionsIds["Taxation Reports"] = "Tables";
		$connectionsIds["system_user"] = "Tables";
		$connectionsIds["fiscal_year"] = "Tables";
		$connectionsIds["beg_bal"] = "Tables";
		$connectionsIds["ewt"] = "Tables";
		$connectionsIds["bank"] = "Tables";
		$connectionsIds["cost_center"] = "Tables";
		$connectionsIds["General Journal Reports"] = "Tables";
		$connectionsIds["Inventory Reports"] = "Tables";
		$connectionsIds["accounting_period"] = "Tables";
		$connectionsIds["bank_recon"] = "Tables";
		$connectionsIds["deposit"] = "Tables";
		$connectionsIds["account_classification"] = "Tables";
		$connectionsIds["chart_of_accounts"] = "Tables";
		$connectionsIds["recon_CR"] = "Tables";
		$connectionsIds["Recon_deposit"] = "Tables";
		$connectionsIds["Recon_CV"] = "Tables";
		$connectionsIds["Recon_GV_dep"] = "Tables";
		$connectionsIds["Recon_GV_CV"] = "Tables";
		$connectionsIds["vat_atc"] = "Tables";
		$connectionsIds["rdo"] = "Tables";
		$connectionsIds["dr"] = "Tables";
		$connectionsIds["purchase_order"] = "Tables";
		$connectionsIds["purchase_order_receiving"] = "Tables";
		$connectionsIds["System_preference"] = "Tables";
		$connectionsIds["ppe_list"] = "Tables";
		$connectionsIds["discounts"] = "Tables";
		$connectionsIds["item_group"] = "Tables";
		$connectionsIds["location"] = "Tables";
		$connectionsIds["meetings"] = "Tables";
		$connectionsIds["calls"] = "Tables";
		$connectionsIds["campaign"] = "Tables";
		$connectionsIds["reimbursement"] = "Tables";
		$connectionsIds["reimbursement_detail"] = "Tables";
		$connectionsIds["Target_customers"] = "Tables";
		$connectionsIds["quotation"] = "Tables";
		$connectionsIds["case"] = "Tables";
		$connectionsIds["production"] = "Tables";
		$connectionsIds["issue"] = "Tables";
		$connectionsIds["issue_detail"] = "Tables";
		$connectionsIds["disposed"] = "Tables";
		$connectionsIds["disposed_detail"] = "Tables";
		$connectionsIds["backload"] = "Tables";
		$connectionsIds["backload_detail"] = "Tables";
		$connectionsIds["vehicle"] = "Tables";
		$connectionsIds["uom"] = "Tables";
		$connectionsIds["trip_ticket"] = "Tables";
		$connectionsIds["approver_list"] = "Tables";
		$connectionsIds["area"] = "Tables";
		$connectionsIds["currency"] = "Tables";
		$connectionsIds["position"] = "Tables";
		$connectionsIds["service_center"] = "Tables";
		$connectionsIds["tax_status"] = "Tables";
		$connectionsIds["competitor"] = "Tables";
		$connectionsIds["campaign_type"] = "Tables";
		$connectionsIds["fixed_asset_return_detail"] = "Tables";
		$connectionsIds["fixed_assets_return"] = "Tables";
		$connectionsIds["Prospect_customers"] = "Tables";
		$connectionsIds["trip_detail"] = "Tables";
		$connectionsIds["beg_baldetail"] = "Tables";
		$connectionsIds["cash_disbursement_detail2"] = "Tables";
		$connectionsIds["case_solutions"] = "Tables";
		$connectionsIds["cash_disbursment_detail"] = "Tables";
		$connectionsIds["cash_receipt_detail"] = "Tables";
		$connectionsIds["cash_receipt_detail2"] = "Tables";
		$connectionsIds["general_journal_detail"] = "Tables";
		$connectionsIds["employee_list"] = "Tables";
		$connectionsIds["petty_cash"] = "Tables";
		$connectionsIds["petty_cash_detail"] = "Tables";
		$connectionsIds["center_replenishment"] = "Tables";
		$connectionsIds["center_replenishment_detail"] = "Tables";
		$connectionsIds["credit_memo"] = "Tables";
		$connectionsIds["credit_memo_detail"] = "Tables";
		$connectionsIds["debit_memo"] = "Tables";
		$connectionsIds["debit_memo_detail"] = "Tables";
		$connectionsIds["project_replenishment"] = "Tables";
		$connectionsIds["project_replenishment_detail"] = "Tables";
		$connectionsIds["business_partner_group"] = "Tables";
		$connectionsIds["Sales_Representatives"] = "Tables";
		$connectionsIds["sales"] = "Tables";
		$connectionsIds["sales_detail"] = "Tables";
		$connectionsIds["item_bill_of_materials"] = "Tables";
		$connectionsIds["item_uom_conversion"] = "Tables";
		$connectionsIds["tranasactions_for_approval"] = "Tables";
		$connectionsIds["deposit_detail"] = "Tables";
		$connectionsIds["purchase_detail"] = "Tables";
		$connectionsIds["dr_detail"] = "Tables";
		$connectionsIds["purchase_order_detail"] = "Tables";
		$connectionsIds["Sales_Order_for_checking"] = "Tables";
		$connectionsIds["sales_order_detail_for_checking"] = "Tables";
		$connectionsIds["Sales_Order_for_approval"] = "Tables";
		$connectionsIds["sales_order_detail_for_approval"] = "Tables";
		$connectionsIds["Quotation_for_approval"] = "Tables";
		$connectionsIds["Quotation_for_checking"] = "Tables";
		$connectionsIds["quotation_detail_for_checking"] = "Tables";
		$connectionsIds["quotation_detail_for_approval"] = "Tables";
		$connectionsIds["Reimbursement_for_checking"] = "Tables";
		$connectionsIds["reimbursement_detail_for_checking"] = "Tables";
		$connectionsIds["Reimbursement_for_approval"] = "Tables";
		$connectionsIds["reimbursement_detail_for_approval"] = "Tables";
		$connectionsIds["Project_Replenishment_for_checking"] = "Tables";
		$connectionsIds["project_replenishment_detail_for_checking"] = "Tables";
		$connectionsIds["Project_Replenishment_for_approval"] = "Tables";
		$connectionsIds["project_replenishment_detail_for_approval"] = "Tables";
		$connectionsIds["center_replenishment_detail_for_checking"] = "Tables";
		$connectionsIds["center_replenishment_detail_for_approval"] = "Tables";
		$connectionsIds["Purchases_Order_for_checking2"] = "Tables";
		$connectionsIds["purchase_order_detail_for_checking"] = "Tables";
		$connectionsIds["Purchases_Order_for_approval2"] = "Tables";
		$connectionsIds["purchase_order_detail_for_approval"] = "Tables";
		$connectionsIds["Receiving_for_checking2"] = "Tables";
		$connectionsIds["purchase_order_receiving_detail_for_checking"] = "Tables";
		$connectionsIds["Receiving_for_approval2"] = "Tables";
		$connectionsIds["purchase_order_receiving_detail_for_approval"] = "Tables";
		$connectionsIds["Delivery_Receipt_for_checking"] = "Tables";
		$connectionsIds["dr_detail_for_checking"] = "Tables";
		$connectionsIds["Delivery_Receipt_for_approval"] = "Tables";
		$connectionsIds["dr_detail_for_approval"] = "Tables";
		$connectionsIds["Production_for_checking"] = "Tables";
		$connectionsIds["production_detail_for_checking"] = "Tables";
		$connectionsIds["Production_for_approval"] = "Tables";
		$connectionsIds["production_detail_for_approval"] = "Tables";
		$connectionsIds["Issue_for_checking"] = "Tables";
		$connectionsIds["supply_issue_detail_for_checking"] = "Tables";
		$connectionsIds["Issue_for_approval"] = "Tables";
		$connectionsIds["supply_issue_detail_for_approval"] = "Tables";
		$connectionsIds["Return_for_checking"] = "Tables";
		$connectionsIds["supply_return_detail_for_checking"] = "Tables";
		$connectionsIds["Return_for_approval"] = "Tables";
		$connectionsIds["supply_return_detail_approval"] = "Tables";
		$connectionsIds["supply_disposed_detail_for_checking"] = "Tables";
		$connectionsIds["Disposal_for_approval"] = "Tables";
		$connectionsIds["supply_disposed_detail_for_approval"] = "Tables";
		$connectionsIds["Fixed_Asset_Issue_for_checking"] = "Tables";
		$connectionsIds["Fixed_Asset_Issue_for_approval"] = "Tables";
		$connectionsIds["fixed_asset_issue_detail_for_approval"] = "Tables";
		$connectionsIds["fixed_asset_issue_detail_for_checking"] = "Tables";
		$connectionsIds["Fixed_Asset_Return_for_checking"] = "Tables";
		$connectionsIds["fixed_asset_return_detail_for_checking"] = "Tables";
		$connectionsIds["Fixed_Asset_Return_for_approval"] = "Tables";
		$connectionsIds["fixed_asset_return_detail_for_approval"] = "Tables";
		$connectionsIds["Petty_Cash_for_checking"] = "Tables";
		$connectionsIds["petty_cash_detail_for_checking"] = "Tables";
		$connectionsIds["Petty_Cash_for_approval"] = "Tables";
		$connectionsIds["petty_cash_detail_for_approval"] = "Tables";
		$connectionsIds["Deposit_for_checking"] = "Tables";
		$connectionsIds["deposit_detail_for_checking"] = "Tables";
		$connectionsIds["Deposit_for_approval"] = "Tables";
		$connectionsIds["deposit_detail_for_approval"] = "Tables";
		$connectionsIds["Credit_Memo_for_checking"] = "Tables";
		$connectionsIds["credit_memo_detail_for_checking"] = "Tables";
		$connectionsIds["Credit_Memo_for_approval"] = "Tables";
		$connectionsIds["credit_memo_detail_for_approval"] = "Tables";
		$connectionsIds["Debit_Memo_for_checking"] = "Tables";
		$connectionsIds["debit_memo_detail_for_checking"] = "Tables";
		$connectionsIds["debit_memo_for_approval"] = "Tables";
		$connectionsIds["debit_memo_detail_for_approval"] = "Tables";
		$connectionsIds["Purchases_for_checking"] = "Tables";
		$connectionsIds["credit_purchase_detail_for_checking"] = "Tables";
		$connectionsIds["Purchases_for_approval"] = "Tables";
		$connectionsIds["credit_purchase_detail_for_approval"] = "Tables";
		$connectionsIds["Sales_for_checking"] = "Tables";
		$connectionsIds["Sales_for_approval"] = "Tables";
		$connectionsIds["credit_sales_detail_for_checking"] = "Tables";
		$connectionsIds["credit_sales_detail_for_approval"] = "Tables";
		$connectionsIds["Cash_Disbursement_for_checking"] = "Tables";
		$connectionsIds["check_voucher_detail_for_checking"] = "Tables";
		$connectionsIds["check_voucher_detail2_for_checking"] = "Tables";
		$connectionsIds["Cash_Disbursement_for_approval"] = "Tables";
		$connectionsIds["check_voucher_detail_for_approval"] = "Tables";
		$connectionsIds["check_voucher_detail2_for_approval"] = "Tables";
		$connectionsIds["document"] = "Tables";
		$connectionsIds["General_Journal_for_checking"] = "Tables";
		$connectionsIds["general_journal_detail_for_checking"] = "Tables";
		$connectionsIds["General_Journal_for_approval"] = "Tables";
		$connectionsIds["general_journal_detail_for_approval"] = "Tables";
		$connectionsIds["for_submissions"] = "Tables";
		$connectionsIds["submission_details"] = "Tables";
		$connectionsIds["sales_order_detail_billing_dates"] = "Tables";
		$connectionsIds["Submission_Search"] = "Tables";
		$connectionsIds["Submission_Reminders"] = "Tables";
		$connectionsIds["years"] = "Tables";
		$connectionsIds["document_type"] = "Tables";
		$connectionsIds["estimate"] = "Tables";
		$connectionsIds["estimate_detail"] = "Tables";
		$connectionsIds["purchase_request"] = "Tables";
		$connectionsIds["purchase_request_detail"] = "Tables";
		$connectionsIds["inv_adj_detail"] = "Tables";
		$connectionsIds["inventory_adj"] = "Tables";
		$connectionsIds["inventory_tranfer"] = "Tables";
		$connectionsIds["inventory_transfer_detail"] = "Tables";
		$connectionsIds["Customers"] = "Tables";
		$connectionsIds["Suppliers"] = "Tables";
		$connectionsIds["Target_contacts"] = "Tables";
		$connectionsIds["Prospect_contacts"] = "Tables";
		$connectionsIds["Customer_Contacts"] = "Tables";
		$connectionsIds["Supplier_Contacts"] = "Tables";
		$connectionsIds["Statement_of_Account"] = "Tables";
		$connectionsIds["Inventory_Adjustment_for_checking"] = "Tables";
		$connectionsIds["inv_adj_detail_for_checking"] = "Tables";
		$connectionsIds["Inventory_Adjustment_for_approval"] = "Tables";
		$connectionsIds["inv_adj_detail_for_approval"] = "Tables";
		$connectionsIds["for_checking"] = "Tables";
		$connectionsIds["for_approval"] = "Tables";
		$connectionsIds["Estimate_for_approval"] = "Tables";
		$connectionsIds["Estimate_for_checking"] = "Tables";
		$connectionsIds["estimate_detail_for_checking"] = "Tables";
		$connectionsIds["estimate_detail_for_approval"] = "Tables";
		$connectionsIds["Follow_up"] = "Tables";
		$connectionsIds["coming_meetings"] = "Tables";
		$connectionsIds["tasks"] = "Tables";
		$connectionsIds["Project_phases"] = "Tables";
		$connectionsIds["origin_fa"] = "Tables";
		$connectionsIds["origin_serialized_supply"] = "Tables";
		$connectionsIds["Prospect_customers_to_Customer"] = "Tables";
		$connectionsIds["production_request"] = "Tables";
		$connectionsIds["production_request_detail"] = "Tables";
		$connectionsIds["department"] = "Tables";
		$connectionsIds["journal_entry_cash_disbursement"] = "Tables";
		$connectionsIds["journal_entry_cash_receipt"] = "Tables";
		$connectionsIds["journal_entry_center_replenishment"] = "Tables";
		$connectionsIds["journal_entry_credit_memo"] = "Tables";
		$connectionsIds["journal_entry_debit_memo"] = "Tables";
		$connectionsIds["journal_entry_depreciation"] = "Tables";
		$connectionsIds["journal_entry_general_journal"] = "Tables";
		$connectionsIds["journal_entry_inventory_adjustment"] = "Tables";
		$connectionsIds["journal_entry_project_replenishment"] = "Tables";
		$connectionsIds["journal_entry_purchases"] = "Tables";
		$connectionsIds["journal_entry_replenishment"] = "Tables";
		$connectionsIds["journal_entry_sales"] = "Tables";
		$connectionsIds["journal_entry_service_revenue"] = "Tables";
		$connectionsIds["Credit_Card_monitoring"] = "Tables";
		$connectionsIds["sales_report_per_item Chart"] = "Tables";
		$connectionsIds["Purchase_Request_for_approval"] = "Tables";
		$connectionsIds["purchase_request_detail_approval"] = "Tables";
		$connectionsIds["Production_Request_for_approval"] = "Tables";
		$connectionsIds["production_request_detail_for_approval"] = "Tables";
		$connectionsIds["Inventory_Transfer_for_approval"] = "Tables";
		$connectionsIds["inventory_transfer_detail_for_approval"] = "Tables";
		$connectionsIds["Purchase_Request_for_checking"] = "Tables";
		$connectionsIds["Production_Request_for_checking"] = "Tables";
		$connectionsIds["Inventory_Transfer_for_checking"] = "Tables";
		$connectionsIds["inventory_transfer_detail_for_checking"] = "Tables";
		$connectionsIds["Backload_for_approval"] = "Tables";
		$connectionsIds["backload_detail_for_approval"] = "Tables";
		$connectionsIds["Backload_for_checking"] = "Tables";
		$connectionsIds["backload_detail_for_checking"] = "Tables";
		$connectionsIds["Cash_Receipt_for_approval"] = "Tables";
		$connectionsIds["Cash_Receipt_for_checking"] = "Tables";
		$connectionsIds["cash_receipt_detail_for_checking"] = "Tables";
		$connectionsIds["cash_receipt_detail_for_approval"] = "Tables";
		$connectionsIds["Service_Report_for_approval"] = "Tables";
		$connectionsIds["Service_Report_for_checking"] = "Tables";
		$connectionsIds["service_report_detail_for_approval"] = "Tables";
		$connectionsIds["service_report_detail_for_checking"] = "Tables";
		$connectionsIds["journal_entry_production"] = "Tables";
		$connectionsIds["journal_entry_deposit"] = "Tables";
		$connectionsIds["production_request_bom"] = "Tables";
		$connectionsIds["production_bom"] = "Tables";
		$connectionsIds["company_info"] = "Tables";
		$connectionsIds["return"] = "Tables";
		$connectionsIds["return_detail"] = "Tables";
		$connectionsIds["fixed_asset_issue"] = "Tables";
		$connectionsIds["fixed_asset_issue_detail"] = "Tables";
		$connectionsIds["production_request_detail_for_checking"] = "Tables";
		$connectionsIds["cash_disbursment_detail_for_approval"] = "Tables";
		$connectionsIds["cash_disbursment_detail_for_checking"] = "Tables";
		$connectionsIds["purchase_request_detail_for_checking"] = "Tables";
		$connectionsIds["trip_ticket_for_approval"] = "Tables";
		$connectionsIds["trip_ticket_for_checking"] = "Tables";
		$connectionsIds["trip_detail_for_approval"] = "Tables";
		$connectionsIds["trip_detail_for_checking"] = "Tables";
		$connectionsIds["sales_report"] = "Tables";
		$connectionsIds["Customer_Balance_Report"] = "Tables";
		$connectionsIds["Customer_Balance_Report_detail"] = "Tables";
		$connectionsIds["Collection_Report_per_Customer"] = "Tables";
		$connectionsIds["Aging_of_AR_Report"] = "Tables";
		$connectionsIds["Aging_of_AR_Report_detail"] = "Tables";
		$connectionsIds["Payment_per_Supplier_Report"] = "Tables";
		$connectionsIds["Aging of_AP_Report"] = "Tables";
		$connectionsIds["Aging of_AP_Report_detail"] = "Tables";
		$connectionsIds["Inventory_Balance_Report"] = "Tables";
		$connectionsIds["Inventory_Valuation_Report"] = "Tables";
		$connectionsIds["Inventory_Below_Reorder_Point_Report"] = "Tables";
		$connectionsIds["Sales_Report_per_BP"] = "Tables";
		$connectionsIds["Sales_Report_per_Item"] = "Tables";
		$connectionsIds["Sales_Report_per_Cost_Center"] = "Tables";
		$connectionsIds["Sales_Report_per_BP_detail"] = "Tables";
		$connectionsIds["Sales_Report_per_Item_detail"] = "Tables";
		$connectionsIds["Sales_Report_per_Cost_Center_detail"] = "Tables";
		$connectionsIds["Purchase_Report"] = "Tables";
		$connectionsIds["Purchase_Report_per_BP"] = "Tables";
		$connectionsIds["Purchase_Report_per_Item"] = "Tables";
		$connectionsIds["Purchase_Report_per_Cost_Center"] = "Tables";
		$connectionsIds["Purchase_Report_per_BP_detail"] = "Tables";
		$connectionsIds["Purchase_Report_per_Item_detail"] = "Tables";
		$connectionsIds["Purchase_Report_per_Cost_Center_detail"] = "Tables";
		$connectionsIds["Sales_Report_per_sales_rep"] = "Tables";
		$connectionsIds["PDC_Registry_Report"] = "Tables";
		$connectionsIds["Available_Vehicles"] = "Tables";
		$connectionsIds["Trip_Ticket_Report"] = "Tables";
		$connectionsIds["Purchase_Request_Report"] = "Tables";
		$connectionsIds["Purchase_Import_Costing"] = "Tables";
		$connectionsIds["Production_Costing_Report"] = "Tables";
		$connectionsIds["Production_Report"] = "Tables";
		$connectionsIds["Closed_Opportunities"] = "Tables";
		$connectionsIds["Cases_with_Sotutions"] = "Tables";
		$connectionsIds["Totals_Cases_Created"] = "Tables";
		$connectionsIds["Totals_Open_Cases_by_Agent"] = "Tables";
		$connectionsIds["Campaign_Contacts"] = "Tables";
		$connectionsIds["Campaign_leads"] = "Tables";
		$connectionsIds["Campaign_ROI_Analysis"] = "Tables";
		$connectionsIds["Campaign_Revenue"] = "Tables";
		$connectionsIds["Opportunity_Schedule"] = "Tables";
		$connectionsIds["Sales_by_Account"] = "Tables";
		$connectionsIds["Sales_by_Lead_Source"] = "Tables";
		$connectionsIds["Sales_by_Partner"] = "Tables";
		$connectionsIds["Sales_by_Rep"] = "Tables";
		$connectionsIds["DR_Reports"] = "Tables";
		$connectionsIds["Sales_Order_Report"] = "Tables";
		$connectionsIds["Case_History"] = "Tables";
		$connectionsIds["quality_control"] = "Tables";
		$connectionsIds["quality_control_detail"] = "Tables";
		$connectionsIds["budget"] = "Tables";
		$connectionsIds["budget_detail"] = "Tables";
		$connectionsIds["repairs_and_maintenance"] = "Tables";
		$connectionsIds["production_form"] = "Tables";
		$connectionsIds["rfp"] = "Tables";
		$connectionsIds["telex"] = "Tables";
		$connectionsIds["repairs_and_maintenance_detail"] = "Tables";
		$connectionsIds["repair_list"] = "Tables";
		$connectionsIds["marketing_colateral"] = "Tables";
		$connectionsIds["email"] = "Tables";
		$connectionsIds["accessories"] = "Tables";
		$connectionsIds["sales_order_materials_detail"] = "Tables";
		$connectionsIds["sales_order_progress_billing_detail"] = "Tables";
		$connectionsIds["sales_order"] = "Tables";
		$connectionsIds["sales_order_detail"] = "Tables";
		$connectionsIds["vat_relief_importation"] = "Tables";
		$connectionsIds["vat_relief_purchases"] = "Tables";
		$connectionsIds["vat_relief_sales"] = "Tables";
		$connectionsIds["journal_ppe_disposal"] = "Tables";
		$connectionsIds["credit_memo_detail_cgs"] = "Tables";
		$connectionsIds["marketing_request_form"] = "Tables";
		$connectionsIds["Quotation_Dashboard_Customer"] = "Tables";
		$connectionsIds["Sales_Order_Dashboard_Customer"] = "Tables";
		$connectionsIds["Delivery_Receipt_Dashboard_Customer"] = "Tables";
		$connectionsIds["Sales_Dashboard_Customer"] = "Tables";
		$connectionsIds["Backload_Dashboard_Customer"] = "Tables";
		$connectionsIds["Purchase_Request_Dashboard_Supplier"] = "Tables";
		$connectionsIds["Purchase_Order_Dashboard_Supplier"] = "Tables";
		$connectionsIds["Purchase_Dashboard_Supplier"] = "Tables";
		$connectionsIds["Cash_Disbursement_Dashboard_Supplier"] = "Tables";
		$connectionsIds["Debit_Memo_Dashboard_Supplier"] = "Tables";
		$connectionsIds["Purchase_Request_Dashboard_Item"] = "Tables";
		$connectionsIds["Purchase_Order_Dashboard_Item"] = "Tables";
		$connectionsIds["Purchase_Order_Receiving_Dashboard_Item"] = "Tables";
		$connectionsIds["Production_Dashboard_Item"] = "Tables";
		$connectionsIds["budget_for_approval"] = "Tables";
		$connectionsIds["budget_for_checking"] = "Tables";
		$connectionsIds["budget_detail_for_checking"] = "Tables";
		$connectionsIds["budget_detail_for_approval"] = "Tables";
		$connectionsIds["request_for_payment_for_approval"] = "Tables";
		$connectionsIds["request_for_payment_for_checking"] = "Tables";
		$connectionsIds["repairs_and_maintenance_detail_for_approval"] = "Tables";
		$connectionsIds["request_for_payment_detail_for_approval"] = "Tables";
		$connectionsIds["request_for_payment_detail_for_checking"] = "Tables";
		$connectionsIds["repairs_and_maintenance_for_approval"] = "Tables";
		$connectionsIds["repairs_and_maintenance_for_checking"] = "Tables";
		$connectionsIds["repairs_and_maintenance_detail_for_checking"] = "Tables";
		$connectionsIds["Disposal_for_checking"] = "Tables";
		$connectionsIds["Active_campaign_report"] = "Tables";
		$connectionsIds["For_Billing_report"] = "Tables";
		$connectionsIds["sales_order_for_project_mngmt"] = "Tables";
		$connectionsIds["inventory_transfer_cost_detail"] = "Tables";
		$connectionsIds["Inventory_per_Cost_center_Report"] = "Tables";
		$connectionsIds["Deposit_Report"] = "Tables";
		$connectionsIds["SO_for_Billing_Report"] = "Tables";
		$connectionsIds["Billing_due_dates_Report"] = "Tables";
		$connectionsIds["Packing_List_Report"] = "Tables";
		$connectionsIds["Open_Trip_Ticket_Report"] = "Tables";
		$connectionsIds["Transfer_Report"] = "Tables";
		$connectionsIds["Production_Report_invw"] = "Tables";
		$connectionsIds["Supply_per_Department__Report_invw"] = "Tables";
		$connectionsIds["Item_Balances_Report_invw"] = "Tables";
		$connectionsIds["Open_Case_Report_s"] = "Tables";
		$connectionsIds["Sales_Order_per_Campaign_Report_marketing"] = "Tables";
		$connectionsIds["Sales_per_campaign_Report_marketing"] = "Tables";
		$connectionsIds["Asset_per_Accountability_Report_invw"] = "Tables";
		$connectionsIds["Deposit_in_transit_Report"] = "Tables";
		$connectionsIds["Outstanding_Checks_Report"] = "Tables";
		$connectionsIds["Attendance_Report"] = "Tables";
		$connectionsIds["Followup_Report"] = "Tables";
		$connectionsIds["Coming_Meeting_Report"] = "Tables";
		$connectionsIds["Meeting_Report"] = "Tables";
		$connectionsIds["Next_Action_Report"] = "Tables";
		$connectionsIds["Calls_Report"] = "Tables";
		$connectionsIds["sales_per_sales_rep_Report"] = "Tables";
		$connectionsIds["Withholding_Tax_Collection_List_per_Month_Report"] = "Tables";
		$connectionsIds["purchase_report_per_item Chart"] = "Tables";
		$connectionsIds["System_Audit_Report"] = "Tables";
		$connectionsIds["sale_cost_detail"] = "Tables";
		$connectionsIds["currency_exchange_rate"] = "Tables";
		$connectionsIds["cash_receipt_cost_detail"] = "Tables";
		$connectionsIds["production_cost_detail"] = "Tables";
		$connectionsIds["Quotation_Dashboard"] = "Tables";
		$connectionsIds["Sales_Order_Dashboard"] = "Tables";
		$connectionsIds["Reimbursement_Dashboard"] = "Tables";
		$connectionsIds["sum_dr_qty"] = "Tables";
		$connectionsIds["check_book_registry"] = "Tables";
		$connectionsIds["check_book_registry_detail"] = "Tables";
		$connectionsIds["request_for_payment"] = "Tables";
		$connectionsIds["request_for_payment_detail"] = "Tables";
		$connectionsIds["vat_type"] = "Tables";
		$connectionsIds["quotation_detail"] = "Tables";
		$connectionsIds["bal_purchase_order_detail"] = "Tables";
		$connectionsIds["bal_sales_detail"] = "Tables";
		$connectionsIds["pending_items_reminders"] = "Tables";
		$connectionsIds["petty_cash_for_liquidation"] = "Tables";
		$connectionsIds["payment_method"] = "Tables";
		$connectionsIds["account_type"] = "Tables";
		$connectionsIds["phase"] = "Tables";
		$connectionsIds["item_cost_recalculation"] = "Tables";
		$connectionsIds["issue_components"] = "Tables";
		$connectionsIds["issue_components_detail"] = "Tables";
		$connectionsIds["demand_plan"] = "Tables";
		$connectionsIds["input_vat_allocation"] = "Tables";
		$connectionsIds["case_type"] = "Tables";
		$connectionsIds["qc_inspection"] = "Tables";
		$connectionsIds["packing_list_for_checking"] = "Tables";
		$connectionsIds["packing_list_for_approval"] = "Tables";
		$connectionsIds["packing_list_detail_for_checking"] = "Tables";
		$connectionsIds["packing_list_detail_for_approval"] = "Tables";
		$connectionsIds["disposed_inv_detail"] = "Tables";
		$connectionsIds["disposed_inv"] = "Tables";
		$connectionsIds["disposed_inv_for_checking"] = "Tables";
		$connectionsIds["disposed_inv_for_approval"] = "Tables";
		$connectionsIds["disposed_inv_detail_for_checking"] = "Tables";
		$connectionsIds["disposed_inv_detail_for_approval"] = "Tables";
		$connectionsIds["For_Approval_Dashboard"] = "Tables";
		$connectionsIds["For_Checking_Dashboard"] = "Tables";
		$connectionsIds["Customer_For_Approval"] = "Tables";
		$connectionsIds["request_for_payment_report"] = "Tables";
		$connectionsIds["credit_memo_report"] = "Tables";
		$connectionsIds["debit_memo_report"] = "Tables";
		$connectionsIds["production_request_report"] = "Tables";
		$connectionsIds["Supplies_Balance_report"] = "Tables";
		$connectionsIds["approved_sales_order"] = "Tables";
		$connectionsIds["purchase_order_for_receiving"] = "Tables";
		$connectionsIds["packing_list"] = "Tables";
		$connectionsIds["packing_list_detail"] = "Tables";
		$connectionsIds["open_quote"] = "Tables";
		$connectionsIds["open_estimates"] = "Tables";
		$connectionsIds["open_po"] = "Tables";
		$connectionsIds["open_pr"] = "Tables";
		$connectionsIds["open_production_request"] = "Tables";
		$connectionsIds["open_qc"] = "Tables";
		$connectionsIds["open_receiving"] = "Tables";
		$connectionsIds["open_request_for_payment"] = "Tables";
		$connectionsIds["open_so"] = "Tables";
		$connectionsIds["open_so_for_invoice"] = "Tables";
		$connectionsIds["open_so_for_project_management"] = "Tables";
		$connectionsIds["open_backload"] = "Tables";
		$connectionsIds["purchase_order_receiving_detail"] = "Tables";
		$connectionsIds["Purchase_Order_Receiving_Dashboard_Supplier"] = "Tables";
		$connectionsIds["avg_inv"] = "Tables";
		$connectionsIds["open_backload_Report"] = "Tables";
		$connectionsIds["open_estimates_Report"] = "Tables";
		$connectionsIds["open_po_Report"] = "Tables";
		$connectionsIds["open_pr_Report"] = "Tables";
		$connectionsIds["open_production_request_Report"] = "Tables";
		$connectionsIds["open_qc_Report"] = "Tables";
		$connectionsIds["open_so_for_project_management_Report"] = "Tables";
		$connectionsIds["open_so_for_invoice_Report"] = "Tables";
		$connectionsIds["open_so_Report"] = "Tables";
		$connectionsIds["open_receiving_Report"] = "Tables";
		$connectionsIds["monthly_consuption_for_production_Report"] = "Tables";
		$connectionsIds["issue_components_for_approval"] = "Tables";
		$connectionsIds["issue_components_for_checking"] = "Tables";
		$connectionsIds["issue_components_detail_for_checking"] = "Tables";
		$connectionsIds["issue_components_detail_for_approval"] = "Tables";
		$connectionsIds["Purchases_Order_for_checking"] = "Tables";
		$connectionsIds["Purchases_Order_for_approval"] = "Tables";
		$connectionsIds["Receiving_for_checking"] = "Tables";
		$connectionsIds["Receiving_for_approval"] = "Tables";
		$connectionsIds["inventory_tranfer_Report"] = "Tables";
		$connectionsIds["Purchase_vs_Production_Report"] = "Tables";
		$connectionsIds["cash_sales_Report"] = "Tables";
		$connectionsIds["Center_Replenishment_for_approval"] = "Tables";
		$connectionsIds["Center_Replenishment_for_checking"] = "Tables";
		$connectionsIds["Purchase_vs_Production"] = "Tables";
		$connectionsIds["Production_Summary"] = "Tables";
		$connectionsIds["pending_production_request"] = "Tables";
		$connectionsIds["quality_control_for_approval"] = "Tables";
		$connectionsIds["quality_control_for_checking"] = "Tables";
		$connectionsIds["quality_control_detail_for_approval"] = "Tables";
		$connectionsIds["raw_cake_balance"] = "Tables";
		$connectionsIds["bal_cashreceipt"] = "Tables";
		$connectionsIds["opportunity"] = "Tables";
		$connectionsIds["Current_Accounts"] = "Tables";
		$connectionsIds["opportunity_details"] = "Tables";
		$connectionsIds["Collection_Report_per_Sales_Rep"] = "Tables";
		$connectionsIds["Inventory_per_lot_no"] = "Tables";
		$connectionsIds["marketing_request_form_for_checking"] = "Tables";
		$connectionsIds["marketing_request_form_for_approval"] = "Tables";
		$connectionsIds["Cashflow_forcast"] = "Tables";
		$connectionsIds["Business_Owner_Sales_Report"] = "Tables";
		$connectionsIds["Business_Owner_Purchase_Report"] = "Tables";
		$connectionsIds["AR_Billing_due_dates_Report"] = "Tables";
		$connectionsIds["AR_Aging_of_AR_Report"] = "Tables";
		$connectionsIds["AR_Top_5_Customer_balances"] = "Tables";
		$connectionsIds["AR_Progress_Billing_Alert"] = "Tables";
		$connectionsIds["Finance_approved_request_for_payment"] = "Tables";
		$connectionsIds["Finance_Cash_flow_forcast"] = "Tables";
		$connectionsIds["Finance_Approved_Employee_Loan"] = "Tables";
		$connectionsIds["AP_Aging_of_Payables"] = "Tables";
		$connectionsIds["Purchasing_Item_below_reorder"] = "Tables";
		$connectionsIds["production_per_item"] = "Tables";
		$connectionsIds["AP_Top_5_Supplier_balances"] = "Tables";
		$connectionsIds["Warehouse_Item_balances"] = "Tables";
		$connectionsIds["DR_per_Item"] = "Tables";
		$connectionsIds["Logistic_Open_Trip_Ticket"] = "Tables";
		$connectionsIds["Logistic_Available_Vehicles"] = "Tables";
		$connectionsIds["Marketing_Open_Campaign"] = "Tables";
		$connectionsIds["Marketing_Sales_per_campaign"] = "Tables";
		$connectionsIds["Support_Open_Case_Report"] = "Tables";
		$connectionsIds["Support_Service_center"] = "Tables";
		$connectionsIds["Sale_Meeting_Report"] = "Tables";
		$connectionsIds["Sales_Next_Action_Report"] = "Tables";
		$connectionsIds["open_quote_Report"] = "Tables";
		$connectionsIds["Open_cases_per_agent"] = "Tables";
		$connectionsIds["Marketing_sales_per_campaing_chart"] = "Tables";
		$connectionsIds["AP_open_po_receiving"] = "Tables";
		$connectionsIds["Production_Inventory_Balance_Report"] = "Tables";
		$connectionsIds["Production_WIP"] = "Tables";
		$connectionsIds["Warehouse_Inventory_Balance_Report"] = "Tables";
		$connectionsIds["logistic_open_inventory_tranfer"] = "Tables";
		$connectionsIds["Sales_Leads"] = "Tables";
		$connectionsIds["Sales_opportunity_Next_Action"] = "Tables";
		$connectionsIds["Sales_Active_opportunity"] = "Tables";
		$connectionsIds["production_detail"] = "Tables";
		$connectionsIds["quality_control_detail_for_checking"] = "Tables";
		$connectionsIds["EWT_Certificates_summay"] = "Tables";
		$connectionsIds["tax_person_status"] = "Tables";
		$connectionsIds["journal_entry_inv_transfer"] = "Tables";
		$connectionsIds["Recon_withdraw"] = "Tables";
		$connectionsIds["lot"] = "Tables";
		$connectionsIds["serial_no"] = "Tables";
		$connectionsIds["request_for_quote"] = "Tables";
		$connectionsIds["request_for_quoter_detail"] = "Tables";
		$connectionsIds["request_for_quote_for_checking"] = "Tables";
		$connectionsIds["request_for_quote_for_approval"] = "Tables";
		$connectionsIds["request_for_quoter_detail_for_checking"] = "Tables";
		$connectionsIds["request_for_quoter_detail_for_approval"] = "Tables";
		$connectionsIds["estimate_cost"] = "Tables";
		$connectionsIds["bal_sales_order_detail"] = "Tables";
		$connectionsIds["Setup_Wizard"] = "Tables";
		$connectionsIds["Overview"] = "Tables";
		$connectionsIds["Setup_Checklist"] = "Tables";
		$connectionsIds["project_bill_of_material"] = "Tables";
		$connectionsIds["project_progress_billing"] = "Tables";
		$connectionsIds["memorized_transactions"] = "Tables";
		$connectionsIds["change_password"] = "Tables";
		$connectionsIds["checklist"] = "Tables";
		$connectionsIds["stage"] = "Tables";
		$connectionsIds["bal_total_po_detail"] = "Tables";
		$connectionsIds["item_production_phase"] = "Tables";
		$connectionsIds["production_bill_of_material"] = "Tables";
		$connectionsIds["production_stage_cost_detail"] = "Tables";
		$connectionsIds["employees"] = "Tables";
		$connectionsIds["sale_assemble_detail"] = "Tables";
		$connectionsIds["Check_Release"] = "Tables";
		$connectionsIds["unused_check_book_registry"] = "Tables";
		$connectionsIds["item_bom_type"] = "Tables";
		$connectionsIds["canvas"] = "Tables";
		$connectionsIds["canvas_detail"] = "Tables";
		$connectionsIds["inventory_adjustment_type"] = "Tables";
		$connectionsIds["Help"] = "Tables";
		$connectionsIds["Global Accounting Systems"] = "Tables";
		$connectionsIds["bank_collection"] = "Tables";
		$connectionsIds["bank_disbursement"] = "Tables";
		$connectionsIds["project_progress"] = "Tables";
		$connectionsIds["inv_fifo"] = "Tables";
		$connectionsIds["inventory_tranfer_request"] = "Tables";
		$connectionsIds["branch_sales"] = "Tables";
		$connectionsIds["branch_sales_detail"] = "Tables";
		$connectionsIds["inventory_transfer__request_detail"] = "Tables";
		$connectionsIds["inventory_tranfer_view"] = "Tables";
		$connectionsIds["inventory_transfer_detail_view"] = "Tables";
		$connectionsIds["import_inv_adj"] = "Tables";
		$connectionsIds["import_inventory_transfer"] = "Tables";
		$connectionsIds["loyalty_points"] = "Tables";
		$connectionsIds["bal_sales_credit_memo"] = "Tables";
		$connectionsIds["expense"] = "Tables";
		$connectionsIds["bal_sales_credit_memo1"] = "Tables";
		$connectionsIds["bal_invoice_credit_memo"] = "Tables";
		$connectionsIds["item_price_per_category"] = "Tables";
		$connectionsIds["item_group_price_per_category"] = "Tables";
		$connectionsIds["report_center_fs"] = "Tables";
		$connectionsIds["report_center_books"] = "Tables";
		$connectionsIds["report_center_sales"] = "Tables";
		$connectionsIds["report_center_tax"] = "Tables";
		$connectionsIds["report_center_ap"] = "Tables";
		$connectionsIds["report_center_ar"] = "Tables";
		$connectionsIds["report_center_purchases"] = "Tables";
		$connectionsIds["report_center_inventory"] = "Tables";
		$connectionsIds["picklist"] = "Tables";
		$connectionsIds["picklist_detail"] = "Tables";
		$connectionsIds["picklist_detail_done"] = "Tables";
		$connectionsIds["projects"] = "Tables";
		$connectionsIds["item_cost_center_detail"] = "Tables";
		$connectionsIds["DR_Dashboard_Report"] = "Tables";
		$connectionsIds["trip_ticket_Dashboard_Report"] = "Tables";
		$connectionsIds["inventory_tranfer_Dashboard_Report"] = "Tables";
		$connectionsIds["project_replenishment_Dashboard_Report"] = "Tables";
		$connectionsIds["picklist_for_approval"] = "Tables";
		$connectionsIds["picklist_for_checking"] = "Tables";
		$connectionsIds["picklist_detail_for_approval"] = "Tables";
		$connectionsIds["picklist_detail_for_checking"] = "Tables";
		$connectionsIds["bal_purchase_debit_memo"] = "Tables";
		$connectionsIds["bal_purchase_head_debit_memo"] = "Tables";
		$connectionsIds["return_components"] = "Tables";
		$connectionsIds["return_components_detail"] = "Tables";
		$connectionsIds["open_production"] = "Tables";
		$connectionsIds["production _imp"] = "Tables";
		$connectionsIds["issue_component_imp"] = "Tables";
		$connectionsIds["request_for_payment_imp"] = "Tables";
		$connectionsIds["fixed_asset_time_usage"] = "Tables";
		$connectionsIds["fixed_asset_time_usage_detail"] = "Tables";
		$connectionsIds["service_visit"] = "Tables";
		$connectionsIds["so_imp"] = "Tables";
		$connectionsIds["open_cases"] = "Tables";
		$connectionsIds["phase_bill_of_materials"] = "Tables";
		$connectionsIds["production_itemized_detail"] = "Tables";
		$connectionsIds["production_search"] = "Tables";
		$connectionsIds["production_stage_detail"] = "Tables";
		$connectionsIds["service_report"] = "Tables";
		$connectionsIds["service_report_detail"] = "Tables";
		$connectionsIds["excise_tax"] = "Tables";
		$connectionsIds["repairs_particular"] = "Tables";
		$connectionsIds["waranty_detail"] = "Tables";
		$connectionsIds["phase_dropdown1"] = "Tables";
		$connectionsIds["phase_dropdown"] = "Tables";
		$connectionsIds["project_name"] = "Tables";
		$connectionsIds["phase_billing"] = "Tables";
		$connectionsIds["new_ticket"] = "Tables";
		$connectionsIds["new_user"] = "Tables";
		$connectionsIds["gas_type"] = "Tables";
		$connectionsIds["inventory_tranfer_mobile"] = "Tables";
		$connectionsIds["inventory_transfer_detail_mobile"] = "Tables";
		$connectionsIds["Item Expiration Report"] = "Tables";
		$connectionsIds["item_with_balances"] = "Tables";
		$connectionsIds["Import_Bank_Statement"] = "Tables";
		$connectionsIds["salesapp"] = "Tables";
		$connectionsIds["bank_list"] = "Tables";
		$connectionsIds["import_sales"] = "Tables";
		$connectionsIds["production_stage_report"] = "Tables";
		$connectionsIds["user_roles"] = "Tables";
		$connectionsIds["check_formats"] = "Tables";
		$connectionsIds["asset_accountability"] = "Tables";
		$connectionsIds["asset_accountability_detail"] = "Tables";
		$connectionsIds["cash_pullout"] = "Tables";
		$connectionsIds["cash_pullout_detail"] = "Tables";
		$connectionsIds["liquidation"] = "Tables";
		$connectionsIds["liquidation_detail"] = "Tables";
		$connectionsIds["loan"] = "Tables";
		$connectionsIds["loan__detail"] = "Tables";
		$connectionsIds["marketing_job_order"] = "Tables";
		$connectionsIds["spoilage"] = "Tables";
		$connectionsIds["spoilage_detail"] = "Tables";
		$connectionsIds["tara_sheet"] = "Tables";
		$connectionsIds["tara_sheet_detail"] = "Tables";
		$connectionsIds["transmittal_slip"] = "Tables";
		$connectionsIds["transmittal_slip_detail"] = "Tables";
		$connectionsIds["spoilage_detail2"] = "Tables";
		$connectionsIds["admin_rights"] = "Tables";
		$connectionsIds["admin_members"] = "Tables";
		$connectionsIds["admin_users"] = "Tables";
		$connectionsIds["open_rfp_to_apv"] = "Tables";
		$connectionsIds["canvass"] = "Tables";
		$connectionsIds["canvass_for_checking"] = "Tables";
		$connectionsIds["canvass_for_approval"] = "Tables";
		$connectionsIds["canvass_to_purchase_order"] = "Tables";
		$connectionsIds["open_cv_for_por"] = "Tables";
		$connectionsIds["open_cv_for_liq"] = "Tables";
		$connectionsIds["open_dr_for_invoice"] = "Tables";
		$connectionsIds["customer_shipping_address"] = "Tables";
		$connectionsIds["region"] = "Tables";
		$connectionsIds["province"] = "Tables";
		$connectionsIds["city"] = "Tables";
		$connectionsIds["barangay"] = "Tables";
		$connectionsIds["journal_entry_liquidation"] = "Tables";

		$this->_tablesConnectionIds = &$connectionsIds;
	}

	/**
	 * Check if It's possible to add to one table's sql query
	 * an sql subquery to another table.
	 * Access doesn't support subqueries from the same table as main.
	 * @param String dataSourceTName1
	 * @param String dataSourceTName2
	 * @return Boolean
	 */
	public function checkTablesSubqueriesSupport( $dataSourceTName1, $dataSourceTName2 )
	{
		$connId1 = $this->_tablesConnectionIds[ $dataSourceTName1 ];
		$connId2 = $this->_tablesConnectionIds[ $dataSourceTName2 ];

		if( $connId1 != $connId2 )
			return false;

		if( $this->_connectionsData[ $connId1 ]["dbType"] == nDATABASE_Access && $dataSourceTName1 == $dataSourceTName2 )
			return false;

		return true;
	}

	/**
	 * Close db connections
    */
	function CloseConnections()
	{
		foreach( $this->cache as $connection )
		{
			if( $connection )
				$connection->close();
		}
	}
}
?>